//**************************************************************************
//
// Title:               STK94/STK594 AVR-FPGA Interface Design 
// Version:             1.2
// Last Updated:        June 9, 2003
// Target:              AT94K05/10/40
//
// Support Hotline:     +1 (408) 436-4119
// Support Email:       fpslic@atmel.com
// Purpose:             This design serves as a8 bit preloadable up counter
//                      with an active low carry-out signal (RCO).
//                      Once enabled, the counter will increment until it 
//                      reaches it's max value and will hold that value
//                      until preloaded (by the AVR) with new a new count start
//                      value.  The RCO serves as the interrupt strobe to the AVR.
//                      
//**************************************************************************

module counter (clock, // Clock input from AVR (GCLK5 or GCLK6)
		reset, // External reset - (tie off w/pull-down resistor attribute)
		enable,// Increment enale -(tie off w/pull-up resisstor attribute) 
		load,  // From AVR_IOSEL0
		awe,   // From AVR_IOWE
		d,     // From AVR Databus
		q,     // To external I/O
		rco);  // To FPGA_INT0

   input        clock;
   input 	reset;
   input 	enable;
   input 	load;
   input 	awe;
   input [7:0] 	d;

   output [7:0] q;
   output 	rco;

   reg [7:0] 	qint;
   reg [7:0] 	q;
   reg 		rco;
   

//**************************************************************************

   
always @ (posedge clock)
begin
    if (reset == 1'b1)
       qint <=  8'h00;                   
    else			 
       begin
	  if (enable == 1'b1)
	     begin
		if ((load == 1'b1) && (awe == 1'b1))
		   begin
		      qint <= d;   // Load 8 LSB's of counter with interrupt count from AVR
		   end
		else						
		   begin
		      if (qint != 8'hFF)
			 begin
			    qint   <=  qint + 1'b1;
			    q <= qint;
			 end // if (qint != 8'hFF)
		   end                                       
	     end
       end
end

always @(posedge clock)
begin
   if (qint == 8'hff) // Use the carry-out to cause an interrupt to the AVR
                rco <= 1'b0;
   else 
                rco <= 1'b1;
end

endmodule