--**************************************************************************
--
-- Title:               STK94/STK594 AVR-FPGA Interface Design 
-- Version:             1.2
-- Last Updated:        June 9, 2003
-- Target:              AT94K05/10/40
--
-- Support Hotline:     +1 (408) 436-4119
-- Support Email:       fpslic@atmel.com
-- Purpose:             This design serves as a 8 bit preloadable up counter
--                      with an active low carry-out signal (RCO).
--                      Once enabled, the counter will increment until it 
--                      reaches it's max value and will hold that value
--                      until preloaded (by the AVR) with new a new count start
--                      value.  The RCO serves as the interrupt strobe to the AVR.
--                      
--**************************************************************************

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;


entity counter is
  port (clock    :in  std_logic;  --  clock input from avr (gclk5 or gclk6)
        reset  :in  std_logic; --  external reset - (tie off w/pull-down resistor attribute)
        enable :in  std_logic; --  increment enale -(tie off w/pull-up resisstor attribute)
        load   :in  std_logic; --  from avr_iosel0
        awe    :in  std_logic;  --  from avr_iowe
        d      :in  std_logic_vector(7 downto 0); --  from avr databus
        q      :out std_logic_vector(7 downto 0); --  to external i/o
        rco    :buffer std_logic); --  to fpga_int0
end counter;

architecture counter_behav of counter is

begin
  
  process (clock, reset, d, enable, load)
    variable qint: std_logic_vector(7 downto 0) := "00000000";
  begin
    if reset='1' then  
      qint:= "00000000";
      rco <= '1';
    else
      if rising_edge(clock) then
        if enable ='1' then
          if load='1' and awe = '1' then
            qint := d;
          else
            if qint /= "11111111" then
              qint:= qint + 1;
              q <= qint;
            else
              qint:= qint;
              q <= qint;
            end if;
          end if;
        end if;
      end if;
          
    if qint = "11111111" then -- use the carry-out to cause an interrupt to the avr
      rco <= '0';
    else
      rco <= '1';
    end if;
      
  end if;
    
end process;

end counter_behav;

                       
