;***************************************************************************
;*
;* Title:               STK594/STK94 AVR-FPGA Interface Design 
;* Version:             1.2
;* Last Updated:        June 9, 2003
;* Target:              AT94K05/10/40
;*
;* Support Hotline:     +1 (408) 436-4119
;* Support Email:       fpslic@atmel.com
;* Purpose:             This design primarily is an interrupt service routine
;*                      to record the number of times an interrupt occurs on
;*                      FPGA_INT0.  Additionally it will output the current interrupt
;*                      count value to PortD for display on the starter board LED's.
;***************************************************************************


;***************************************************************************
;*                              Include Files
;***************************************************************************

.nolist
.include "AT94KDEF.INC"
.list

;***************************************************************************
;*                             Global Registers
;***************************************************************************

.def    rTemp           = r16           ;Temporary Storage Register
.def    rCount0         = r17           ;Count Register for FPGA Interrupt 0
.def    delay1          = r18           ;delay register
.def    delay2          = r19           ;delay register
.def    delay3          = r20           ;delay register
.def    delay4          = r21           ;delay register

;***************************************************************************
;*                             Interrupt Vectors
;***************************************************************************

        jmp     RESET                   ;Reset Handle: Program Execution Starts Here
        jmp     FPGA_INT0               ;FPGA Interrupt0 Handle
        jmp     EXT_INT0                ;External Interrupt0 Handle
        jmp     FPGA_INT1               ;FPGA Interrupt1 Handle
        jmp     EXT_INT1                ;External Interrupt1 Handle
        jmp     FPGA_INT2               ;FPGA Interrupt2 Handle
        jmp     EXT_INT2                ;External Interrupt2 Handle
        jmp     FPGA_INT3               ;FPGA Interrupt3 Handle
        jmp     EXT_INT3                ;External Interrupt3 Handle
        jmp     TIM2_COMP               ;Timer/Counter2 Compare Match Interrupt Handle
        jmp     TIM2_OVF                ;Timer/Counter2 Overflow Interrupt Handle
        jmp     TIM1_CAPT               ;Timer/Counter1 Capture Event Interrupt Handle
        jmp     TIM1_COMPA              ;Timer/Counter1 Compare Match A Interrupt Handle
        jmp     TIM1_COMPB              ;Timer/Counter1 Compare Match B Interrupt Handle
        jmp     TIM1_OVF                ;Timer/Counter1 Overflow Interrupt Handle
        jmp     TIM0_COMP               ;Timer/Counter0 Compare Match Interrupt Handle
        jmp     TIM0_OVF                ;Timer/Counter0 Overflow Interrupt Handle
        jmp     FPGA_INT4               ;FPGA Interrupt4 Handle
        jmp     FPGA_INT5               ;FPGA Interrupt5 Handle
        jmp     FPGA_INT6               ;FPGA Interrupt6 Handle
        jmp     FPGA_INT7               ;FPGA Interrupt7 Handle
        jmp     UART0_RXC               ;UART0 Receive Complete Interrupt Handle
        jmp     UART0_DRE               ;UART0 Data Register Empty Interrupt Handle
        jmp     UART0_TXC               ;UART0 Transmit Complete Interrupt Handle
        jmp     FPGA_INT8               ;FPGA Interrupt8 Handle
        jmp     FPGA_INT9               ;FPGA Interrupt9 Handle
        jmp     FPGA_INT10              ;FPGA Interrupt10 Handle
        jmp     FPGA_INT11              ;FPGA Interrupt11 Handle
        jmp     UART1_RXC               ;UART1 Receive Complete Interrupt Handle
        jmp     UART1_DRE               ;UART1 Data Register Empty Interrupt Handle
        jmp     UART1_TXC               ;UART1 Transmit Complete Interrupt Handle
        jmp     FPGA_INT12              ;FPGA Interrupt12 Handle
        jmp     FPGA_INT13              ;FPGA Interrupt13 Handle
        jmp     FPGA_INT14              ;FPGA Interrupt14 Handle
        jmp     FPGA_INT15              ;FPGA Interrupt15 Handle
        jmp     TWS_INT                 ;TWS Serial Interrupt Handle

FPGA_INT0:
        inc     rCount0                 ;Increment Count of FPGA Interrupt 0 Calls

					; ************ Important ****************
	com 	rCount0			; If using STK594 + STK500, take the one's completment
						; the of interrupt count  before outputing to PortD. 
						; If using STK94 comment out the "com rCount0" line.
        out     PORTD, rCount0      ; Output Interrupt Count to PORTD

delay:					;Wait a while so count remains visible
		inc	delay4
		cpi	delay4, 0x1f
		brne	delay
		clr	delay4
		inc	delay3
		cpi	delay3, 0xff
		brne	delay
		clr	delay3
		inc 	delay2			
		cpi 	delay2, 0xff
		brne 	delay
		clr	delay2
		inc 	delay1
		cpi 	delay1, 0xff
		clr	delay1
		brne 	delay	

	clr 	rTemp                   ;Prepare to reset FPGA count to zero      
  	cbi 	FISCR, XFIS0            ;Configure FPGA I/O Select Line
    	cbi 	FISCR, XFIS1            ;Configure FPGA I/O Select Line
    	out 	FISUA, rTemp          	;Output Data to FPGA via I/O Select 0
	com		rCount0
      reti				;Go back to main and wait for the next interrupt

EXT_INT0:
        reti
FPGA_INT1:
        reti
EXT_INT1:
        reti
FPGA_INT2:
        reti
EXT_INT2:
        reti
FPGA_INT3:
        reti
EXT_INT3:
        reti
TIM2_COMP:
        reti
TIM2_OVF:
        reti
TIM1_CAPT:
        reti
TIM1_COMPA:
        reti
TIM1_COMPB:
        reti
TIM1_OVF:
        reti
TIM0_COMP:
        reti
TIM0_OVF:
        reti
FPGA_INT4:
        reti
FPGA_INT5:
        reti
FPGA_INT6:
        reti
FPGA_INT7:
        reti
UART0_RXC:
        reti
UART0_DRE:
        reti
UART0_TXC:
        reti
FPGA_INT8:
        reti
FPGA_INT9:
        reti
FPGA_INT10:
        reti
FPGA_INT11:
        reti
UART1_RXC:
        reti
UART1_DRE:
        reti
UART1_TXC:
        reti
FPGA_INT12:
        reti
FPGA_INT13:
        reti
FPGA_INT14:
        reti
FPGA_INT15:
        reti
TWS_INT:
        reti

RESET:
        cli                             ;Disable Global Interrupts

        clr     rCount0                 ;Initialize rCount0 = $00

        ldi     rTemp, high($0FFF)      ;MSB of Last SRAM Address
        out     SPH, rTemp              ;Initialize MSB of Stack Pointer
        ldi     rTemp, low($0FFF)       ;LSB of Last SRAM Address
        out     SPL, rTemp              ;Initialize LSB of Stack Pointer


        ser     rTemp                   ;Set rTemp = $FF
        out     DDRD, rTemp             ;Configure PORTD as Output
	out 	PORTD, rTemp		;Turn off STK594's LED's (on for STK94)
        sbi     FISCR, FIADR            ;Enable FPGA Interrupt/Mask Access
        sbi     FISUA, FINT0            ;Enable FPGA Interrupt0
        cbi     FISCR, FIADR            ;Disable FPGA Interrupt/Mask Access
        sei                             ;Enable Global Interrupts

MAIN:
        rjmp MAIN                       ;Do Nothing, Wait for an Interrupt
